# convert-comma-constraints
import logging
from charmworld.migrations.migrate import setup_mongo_indices
from charmworld.models import (
    convert_bundle_constraints,
    store_bundles,
)


def exodus_update(source, target, current_version):
    """Complete this function with work for an exodus.

    source is the original CharmSource.
    target is the CharmSource to be used after the exodus.
    current_version is the version of this exodus.
    """
    source_db = source.collection.database
    target_db = target.collection.database

    log = logging.getLogger('charm.exodus026')
    log.warning('Starting')
    charms = target.collection
    setup_mongo_indices(charms)
    log.warning('Indices built')

    log.warning('Converting bundles')
    bundles = source_db.bundles.find()
    for bundle_data in bundles:
        if 'inherits' in bundle_data['data']:
            del bundle_data['data']['inherits']
        try:
            convert_bundle_constraints(bundle_data['data'])
        except ValueError as e:
            # This bundle has constraints that are invalid or no longer
            # supported.  Simply skip the bundle.
            log.warning('Bundle has invalid constraint.  Skipping {}'.format(
                bundle_data['_id']))
            log.warning(e.message)
            target_db.bundles.remove(bundle_data['_id'])
            continue
        basket_id = '{}/{}'.format(
            bundle_data['basket_name'],
            bundle_data['basket_revision'])
        deployer_config = {bundle_data['name']: bundle_data['data']}
        store_bundles(
            target_db.bundles,
            deployer_config,
            bundle_data['owner'],
            basket_id,
            bundle_data['first_change'],
            bundle_data['last_change'],
            bundle_data['changes'],
            bundle_data['branch_spec'],
            None,
            target.index_client,
            bundle_data.get('promulgated', False))
    log.warning('Copying charms')
    for charm in source_db.charms.find():
        target.save(charm)
    log.warning('Finished')
