# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

# -*- coding: utf-8 -*-
"""
Load qa questions into the collection for populating the scoring parts of
the quality assessment form.

"""


def iter_categories():
    """Complete this function with work to be done for the migration/update.

    db is the pymongo db instance for our datastore. Charms are in db.charms
    for instance.
    """

    # A list of tuples for question categories.
    # (name, description, questions)
    # where questions = (description, points, extended_description)
    app_armor_url = 'https://help.ubuntu.com/12.04/serverguide/apparmor.html'
    initial = [
        (
            u'data_handling',
            u'Data Handling',
            [
                (u"Handles the service's user data", 1,
                 u"Gracefully handle the service's user data so that the user "
                  "doesn't have to manage it separately."),

                (u'Provides backup mechanism', 1,
                 u"Provide a backup mechanism to the service's user data, "
                  "such as a relationship to a backup subordinate charm, "
                  "snapshots to a bucket, or other means of getting the data "
                  "off of the instance."),

                (u'Provides a restore mechanism', 1,
                 u"Provide a restore mechanism to the service's user data, "
                  "such as a relationship to a backup subordinate charm, "
                  "restoring from a bucket, or other means of getting data "
                  "onto the instance."),

                (u'Provides encryption', 1,
                 u'Provides encryption of service user data.'),
            ],
        ),

        (
            u'secure',
            u'Secure',
            [
                (u'Contains a well-tested AppArmor profile', 1,
                 u'These can be provided by the package itself.'
                  'See the <a href="{}">AppArmor</a> documentation '
                  'for more details.'.format(app_armor_url)),

                (u"Doesn't run as root", 1,
                 u'The service should not run as root. Refer to the '
                  '<a href="http://upstart.ubuntu.com/cookbook/'
                  '#run-a-job-as-a-different-user">'
                  'Upstart documentation</a> for tips on how to do this.'),

                (u'Per instance or service access control', 1,
                 u'Accept relationships only from trusted instances and/or '
                  'services.'),

                (u'Defaults to secure communication', 1,
                 u'Defaults to secure channels when communicating with other '
                  'services and/or multiple units of the same service.'),
            ],
        ),

        (
            u'reliable',
            u'Reliable',
            [
                (u'Fails gracefully if upstream source goes missing', 1,
                 u'Sometimes URLs change or upstream services are not '
                  'available.'),

                (u'Contains a suite of integration tests with the charm '
                  'that pass',
                 1,
                 u'We provide <a href="howto-amulet.html">Amulet</a> '
                  'to help you do this.'),

                (u'Configuration options have safe defaults', 1, u''),
            ]
        ),

        (
            u'scaleable',
            u'Scaleable',
            [
                (u"Responds to add-unit based on the service's needs", 1,
                 u'Users should be able to <tt>juju add-unit</tt> to your '
                  'service and have it scale horizontally.'),

                (u"Responds to remove-unit based on the service's needs", 1,
                 u'Users should be able to <tt>juju remove-unit</tt> to '
                  'your service and have it scale down.'),

                (u'Reuses existing charms for supporting services', 1,
                 u'If the service needs relationships to other services it '
                  'should reuse existing charms from the charm store instead '
                  'of bundling its own.'),
                (u'Monitoring', 1,
                 u'Has relationships to allow the service to be monitored '
                  'by any existing monitoring charm.'),
                (u'Remote logging', 1,
                 u'Has relationships to allow the service to remote log.'),
            ]
        ),


        (
            u'upstream_friendly',
            u'Upstream Friendly',
            [
                (u'Follow deployment recommendations from upstream best '
                  'practices',
                 1,
                 u'Most services have a known-good recommendation from the '
                  'project itself, these should be available for users'),

                (u'Provide up to date versions of the upstream release', 1,
                 u'Provide a config option to allow the user to run newer '
                  'versions of the service.')
            ]
        ),

        (
            u'flexible',
            u'Flexible',
            [
                (u'Exposes version of service as a config option to allow '
                  'easy upgrades',
                 1,
                 u'Allows users to maintain their deployments without '
                  'having to redeploy.'),

                (u'Adheres to the coding guidelines of the language your '
                  'charm is written',
                 1,
                 u''),

                (u'Exposed configurations are subsets of opinionated '
                  'deployments',
                 1,
                 u'Don\'t expose every service option as configuration. '
                  'Make sets of decisions and expose those as one '
                  'configuration option.  For example "fast" vs. "default" '
                  'vs. "slow-and-safer".'),

                (u'Allow installation from the Ubuntu repository',
                 1,
                 u'Should be the default if the service is in Ubuntu.'),

                (u'Allow installation from pure upstream source',
                 1,
                 u'Sometimes people prefer to deploy from pure upstream'),

                (u'Allow installation from your local source',
                 1,
                 u'Sometimes people prefer to deploy from their vetted '
                  'local source.'),

                (u'Allow installation from PPA or other repository if '
                  'available',
                 1,
                 u'Some upstreams run their own repositories '
                  'that they gate and support.  Users should have an option '
                  'of using these.')
            ]
        ),

        (
            u'easy_to_deploy',
            u'Easy to Deploy',
            [
                (u'README with examples of use for a typical workload', 1,
                 u''),
                (u'README with examples of use for workloads at scale', 1,
                 u''),
                (u'README with examples of use of recommended best-practice '
                  'relationships',
                 1,
                 u'')
            ]
        ),
    ]

    """Add the sample data into the db."""
    for name, description, questions in initial:
        category_dict = {
            'name': name,
            'description': description,
            'questions': [{
                'id': '{0}_{1}'.format(name.lower(), i),
                'description': question[0],
                'points': question[1],
                'extended_description': question[2],
            } for i, question in enumerate(questions)]
        }
        yield category_dict
