# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from __future__ import print_function

__metaclass__ = type

import sys
from charmworld.utils import (
    delete_charm,
    get_ini,
)
from charmworld.models import (
    getconnection,
    getdb,
)
from charmworld.search import ElasticSearchClient


CHARM_ID_MESSAGE = (
    'Please provide a charm ID, including revision, e.g., '
    '~charmers/precise/apache2/14')


def main(argv, stderr=sys.stderr, db=None, index_client=None):
    """The main function for the charm removal script."""
    if len(argv) != 2:
        print(
            'Please provide the ID of the charm you wish to remove.',
            file=stderr)
        return 1

    charm_id = argv[1]

    if db is None:
        settings = get_ini()
        connection = getconnection(settings)
        db = getdb(connection, settings.get('mongo.database'))

    # If the specified charm does not exist, tell the user.
    charm_query = {'_id': charm_id}
    charm_data = db.charms.find_one(charm_query)
    if charm_data is None:
        print('Unknown charm ID: %r' % charm_id, file=stderr)
        print(CHARM_ID_MESSAGE, file=stderr)
        return 1

    if index_client is None:
        index_client = ElasticSearchClient.from_settings(settings)

    delete_charm(charm_data, db, index_client)
    return 0


def entry_point():
    """Script entry point for removing a server's start up timestamp."""
    sys.exit(main(sys.argv))
