# Copyright 2012, 2013 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__metaclass__ = type

import sys
from charmworld.utils import (
    get_ini,
    SERVER_STARTED,
)
from charmworld.models import (
    getconnection,
    getdb,
)


def remove_server_start_time_entries(argv, output, db=None):
    """The main function for the server start timestamp removal script."""
    if db is None:
        settings = get_ini()
        connection = getconnection(settings)
        db = getdb(connection, settings.get('mongo.database'))

    if len(argv) < 2:
        print >>output, "At least one server hostname must be specified."
        return 1

    server_start_times = db.server_status.find_one({'_id': SERVER_STARTED})
    if server_start_times is None:
        print >>output, "No server start times are yet recorded."
        return 1

    for name in argv[1:]:
        if name not in server_start_times:
            print >>output, "hostname %s not found." % name
            return 1

    for name in argv[1:]:
        db.server_status.update(
            {
                '_id': SERVER_STARTED,
            },
            {
                '$unset': {name: ''},
            })
    return 0


def entry_point():
    """Script entry point for removing a server's start up timestamp."""
    sys.exit(remove_server_start_time_entries(sys.argv, sys.stderr))
