# Copyright 2012, 2013 Canonical

# Ltd.  This software is licensed under the GNU Affero General Public License
# version 3 (see the file LICENSE).

__metaclass__ = type


from charmworld.testing import factory
from charmworld.utils import build_metric_key
from charmworld.models import (
    DatedMetric,
    DatedMetricSource,
)
from charmworld.views.tests.test_api import (
    APITestBase,
    API3Mixin,
)
from pyramid.httpexceptions import HTTPNotFound


class TestAPI3BundleMetrics(API3Mixin, APITestBase):
    """Test API 3's "charms" endpoint."""

    def test_incrementing_bundle_metric_count(self):
        # The API provides a way to increment a metric count.
        bundle = factory.make_bundle(self.db)[0]
        response = self.get_response(
            'bundle', bundle.id + '/metric/deployments/increment')
        self.assertEqual(200, response.status_code)

        # Now look up the metric that was supposed to be incremented...
        source = DatedMetricSource.from_db(self.db)
        metric_key = build_metric_key('deployments', bundle.id)
        metric = source.retrieve(metric_key)
        # ...and check that it has the right value.
        self.assertEqual(metric.get_total(), 1)

    def test_fetching_bundle_metric_count(self):
        # The API provides a way to retrieve a metric count.
        bundle = factory.make_bundle(self.db)[0]
        metric = DatedMetric()
        metric.increment()
        metric.increment()
        source = DatedMetricSource.from_db(self.db)
        source.store(metric, build_metric_key('deployments', bundle.id))
        response = self.get_response(
            'bundle', bundle.id + '/metric/deployments/total')
        self.assertEqual(200, response.status_code)
        # ...and check that it has the right value.
        self.assertEqual(response.body, '2')

    def test_fetching_unknown_bundle_metric_type_generates_a_404(self):
        # If a metric type is unknown, requesting it generates a 404.
        bundle = factory.make_bundle(self.db)[0]
        with self.assertRaises(HTTPNotFound):
            self.get_response(
                'bundle', bundle.search_id + '/metric/does-not-exist/total')

    def test_requesting_unknown_operation_generates_a_404(self):
        # If an operation is unknown, requesting it generates a 404.
        bundle = factory.make_bundle(self.db)[0]
        metric_key = bundle.search_id + '/metric/deployments/do-something-else'
        with self.assertRaises(HTTPNotFound):
            self.get_response('bundle', metric_key)

    def test_not_including_a_metric_name(self):
        # If no metric name is included, a 404 is generated.
        bundle = factory.make_bundle(self.db)[0]
        with self.assertRaises(HTTPNotFound):
            self.get_response('bundle', bundle.id + '/metric')

    def test_sending_no_metric_operation_generates_a_404(self):
        # If a metrics request does not provide an operation, a 404 is
        # generated.
        bundle = factory.make_bundle(self.db)[0]
        with self.assertRaises(HTTPNotFound):
            self.get_response('bundle', bundle.id + '/metric/deployments')
